@@ -1,4 +1,4 @@
-"""Test the prototype loop extension."""
+"""Test the 1.3 loop feature."""
 
 import json
 from io import StringIO
@@ -10,9 +10,9 @@ from .util import get_data
 
 
 def test_validate_loop() -> None:
-    """Affirm that a loop workflow validates with --enable-ext."""
+    """Affirm that a loop workflow validates with --enable-dev."""
     params = [
-        "--enable-ext",
+        "--enable-dev",
         "--validate",
         get_data("tests/loop/single-var-loop.cwl"),
     ]
@@ -20,7 +20,7 @@ def test_validate_loop() -> None:
 
 
 def test_validate_loop_fail_no_ext() -> None:
-    """Affirm that a loop workflow does not validate when --enable-ext is missing."""
+    """Affirm that a loop workflow does not validate when --enable-dev is missing."""
     params = [
         "--validate",
         get_data("tests/loop/single-var-loop.cwl"),
@@ -31,88 +31,70 @@ def test_validate_loop_fail_no_ext() -> None:
 def test_validate_loop_fail_scatter() -> None:
     """Affirm that a loop workflow does not validate if scatter and loop directives are on the same step."""
     params = [
-        "--enable-ext",
+        "--enable-dev",
         "--validate",
         get_data("tests/loop/invalid-loop-scatter.cwl"),
     ]
     assert main(params) == 1
 
 
-def test_validate_loop_fail_when() -> None:
-    """Affirm that a loop workflow does not validate if when and loop directives are on the same step."""
-    params = [
-        "--enable-ext",
-        "--validate",
-        get_data("tests/loop/invalid-loop-when.cwl"),
-    ]
-    assert main(params) == 1
-
-
 def test_validate_loop_fail_no_loop_when() -> None:
-    """Affirm that a loop workflow does not validate if no loopWhen directive is specified."""
+    """Affirm that a loop workflow does not validate if no 'when' directive is specified."""
     params = [
-        "--enable-ext",
+        "--enable-dev",
         "--validate",
         get_data("tests/loop/invalid-no-loopWhen.cwl"),
     ]
     assert main(params) == 1
 
 
-def test_validate_loop_fail_on_workflow() -> None:
-    """Affirm that a workflow does not validate if it contains a Loop requirement."""
+def test_loop_fail_loop_when_exception() -> None:
+    """Affirm that a loop workflow fails if 'when' directive throws an exception."""
     params = [
-        "--enable-ext",
-        "--validate",
-        get_data("tests/loop/invalid-loop-workflow.cwl"),
+        "--enable-dev",
+        get_data("tests/loop/invalid-loop-when-exception.cwl"),
+        get_data("tests/loop/two-vars-loop-job.yml"),
     ]
     assert main(params) == 1
 
 
-def test_validate_loop_fail_on_command_line_tool() -> None:
-    """Affirm that a CommandLineTool does not validate if it contains a Loop requirement."""
+def test_loop_fail_loop_when_exception_second_iteration() -> None:
+    """Affirm that a loop workflow fails if when directive throws an
+    exception on second iteration."""
     params = [
-        "--enable-ext",
-        "--validate",
-        get_data("tests/loop/invalid-loop-command-line-tool.cwl"),
-    ]
-    assert main(params) == 1
-
-
-def test_validate_loop_fail_on_expression_tool() -> None:
-    """Affirm that an ExpressionTool does not validate if it contains a Loop requirement."""
-    params = [
-        "--enable-ext",
-        "--validate",
-        get_data("tests/loop/invalid-loop-expression-tool.cwl"),
-    ]
-    assert main(params) == 1
-
-
-def test_validate_loop_fail_on_hint() -> None:
-    """Affirm that a loop workflow does not validate if it contains a Loop hint."""
-    params = [
-        "--enable-ext",
-        "--validate",
-        get_data("tests/loop/invalid-loop-hint.cwl"),
+        "--enable-dev",
+        get_data("tests/loop/invalid-loop-when-exception2.cwl"),
+        get_data("tests/loop/two-vars-loop-job.yml"),
     ]
     assert main(params) == 1
 
 
 def test_loop_fail_non_boolean_loop_when() -> None:
-    """Affirm that a loop workflow fails if loopWhen directive returns a non-boolean value."""
+    """Affirm that a loop workflow fails if 'when' directive returns a non-boolean value."""
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/invalid-non-boolean-loopWhen.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
     assert main(params) == 1
 
 
+def test_loop_fail_non_boolean_loop_second_when() -> None:
+    """Affirm that a loop workflow fails if 'when' directive returns
+    a non-boolean value on the second iteration."""
+    params = [
+        "--enable-dev",
+        get_data("tests/loop/invalid-non-boolean-loopWhen2.cwl"),
+        get_data("tests/loop/two-vars-loop-job.yml"),
+    ]
+    assert main(params) == 1
+
+
 def test_loop_single_variable() -> None:
     """Test a simple loop case with a single variable."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/single-var-loop.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
@@ -122,10 +104,10 @@ def test_loop_single_variable() -> None:
 
 
 def test_loop_single_variable_no_iteration() -> None:
-    """Test a simple loop case with a single variable and a false loopWhen condition."""
+    """Test a simple loop case with a single variable and a false 'when' condition."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/single-var-loop-no-iteration.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
@@ -138,7 +120,7 @@ def test_loop_two_variables() -> None:
     """Test a loop case with two variables, which are both back-propagated between iterations."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/two-vars-loop.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
@@ -151,7 +133,7 @@ def test_loop_two_variables_single_backpropagation() -> None:
     """Test loop with 2 variables, but when only one of them is back-propagated between iterations."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/two-vars-loop-2.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
@@ -164,7 +146,7 @@ def test_loop_with_all_output_method() -> None:
     """Test a loop case with outputMethod set to all."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/all-output-loop.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
@@ -174,10 +156,10 @@ def test_loop_with_all_output_method() -> None:
 
 
 def test_loop_with_all_output_method_no_iteration() -> None:
-    """Test a loop case with outputMethod set to all and a false loopWhen condition."""
+    """Test a loop case with outputMethod set to all and a false 'when' condition."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/all-output-loop-no-iteration.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
@@ -190,7 +172,7 @@ def test_loop_value_from() -> None:
     """Test a loop case with a variable generated by a valueFrom directive."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/value-from-loop.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
@@ -202,7 +184,7 @@ def test_loop_value_from() -> None:
 def test_loop_value_from_fail_no_requirement() -> None:
     """Test workflow loop fails for valueFrom without StepInputExpressionRequirement."""
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/invalid-value-from-loop-no-requirement.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
@@ -213,7 +195,7 @@ def test_loop_inside_scatter() -> None:
     """Test a loop subworkflow inside a scatter step."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/loop-inside-scatter.cwl"),
         get_data("tests/loop/loop-inside-scatter-job.yml"),
     ]
@@ -226,7 +208,7 @@ def test_scatter_inside_loop() -> None:
     """Test a loop workflow with inside a scatter step."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/scatter-inside-loop.cwl"),
         get_data("tests/loop/loop-inside-scatter-job.yml"),
     ]
@@ -235,11 +217,24 @@ def test_scatter_inside_loop() -> None:
     assert json.loads(stream.getvalue()) == expected
 
 
+def test_loop_opt_variable() -> None:
+    """Test a loop case with two variables but one is optional."""
+    stream = StringIO()
+    params = [
+        "--enable-dev",
+        get_data("tests/loop/opt-var-loop.cwl"),
+        get_data("tests/loop/single-var-loop-job.yml"),
+    ]
+    main(params, stdout=stream)
+    expected = {"o1": 10}
+    assert json.loads(stream.getvalue()) == expected
+
+
 def test_nested_loops() -> None:
     """Test a workflow with two nested loops."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/loop-inside-loop.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
@@ -252,7 +247,7 @@ def test_nested_loops_all() -> None:
     """Test a workflow with two nested loops, both with outputMethod set to all."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/loop-inside-loop-all.cwl"),
         get_data("tests/loop/two-vars-loop-job.yml"),
     ]
@@ -265,7 +260,7 @@ def test_multi_source_loop_input() -> None:
     """Test a loop with two sources, which are selected through a pickValue directive."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/multi-source-loop.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
@@ -277,7 +272,7 @@ def test_multi_source_loop_input() -> None:
 def test_multi_source_loop_input_fail_no_requirement() -> None:
     """Test that a loop with two sources fails without MultipleInputFeatureRequirement."""
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/invalid-multi-source-loop-no-requirement.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
@@ -288,7 +283,7 @@ def test_default_value_loop() -> None:
     """Test a loop whose source has a default value."""
     stream = StringIO()
     params = [
-        "--enable-ext",
+        "--enable-dev",
         get_data("tests/loop/default-value-loop.cwl"),
         get_data("tests/loop/single-var-loop-job.yml"),
     ]
