@@ -101,6 +101,62 @@ expression: '{"bar": $(inputs.foo.location)}'
 outputs: []
 """
 
+script_int = """
+#!/usr/bin/env cwl-runner
+
+cwlVersion: v1.0
+class: ExpressionTool
+
+inputs:
+  foo: int[]
+
+expression: '{"bar": $(inputs.foo)}'
+
+outputs: []
+"""
+
+script_long = """
+#!/usr/bin/env cwl-runner
+
+cwlVersion: v1.0
+class: ExpressionTool
+
+inputs:
+  foo: long[]
+
+expression: '{"bar": $(inputs.foo)}'
+
+outputs: []
+"""
+
+script_float = """
+#!/usr/bin/env cwl-runner
+
+cwlVersion: v1.0
+class: ExpressionTool
+
+inputs:
+  foo: float[]
+
+expression: '{"bar": $(inputs.foo)}'
+
+outputs: []
+"""
+
+script_double = """
+#!/usr/bin/env cwl-runner
+
+cwlVersion: v1.0
+class: ExpressionTool
+
+inputs:
+  foo: double[]
+
+expression: '{"bar": $(inputs.foo)}'
+
+outputs: []
+"""
+
 scripts_argparse_params = [
     ("help", script_a, lambda x: ["--debug", x, "--input", get_data("tests/echo.cwl")]),
     ("boolean", script_b, lambda x: [x, "--help"]),
@@ -120,6 +176,31 @@ scripts_argparse_params = [
         script_e,
         lambda x: [x, "--foo", "http://example.com"],
     ),
+    (
+        "foo with int",
+        script_int,
+        lambda x: [x, "--foo", "1", "--foo", "2"],
+    ),
+    (
+        "foo with long for large value",
+        script_long,
+        lambda x: [x, "--foo", str(2**31 + 10)],
+    ),
+    (
+        "foo with long for small value",
+        script_long,
+        lambda x: [x, "--foo", str(-1 * (2**31) - 10)],
+    ),
+    (
+        "foo with float",
+        script_float,
+        lambda x: [x, "--foo", "1.2", "--foo", "3.4"],
+    ),
+    (
+        "foo with double",
+        script_double,
+        lambda x: [x, "--foo", "1.2", "--foo", "3.4"],
+    ),
 ]
 
 
